@@ -1,34 +1,15 @@
-# Copyright (c) 2023, Riverbank Computing Limited
-# All rights reserved.
-#
-# This copy of SIP is licensed for use under the terms of the SIP License
-# Agreement.  See the file LICENSE for more details.
-#
-# This copy of SIP may also used under the terms of the GNU General Public
-# License v2 or v3 as published by the Free Software Foundation which can be
-# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-2-Clause
+
+# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
 
 
 from functools import partial
 import os
 
-from ply import lex, yacc
-
-from ...bindings_configuration import get_bindings_configuration
-from ...exceptions import UserException
+from ...exceptions import deprecated, UserException
+from ...module import get_latest_version
 
+from ..bindings_configuration import get_bindings_configuration
 from ..error_log import ErrorLog
 from ..instantiations import instantiate_class
 from ..python_slots import invalid_global_slot, slot_name_detail_map
@@ -47,14 +28,14 @@ from ..utils import (argument_as_str, cached_name, find_iface_file,
 from . import rules
 from . import tokens
 from .annotations import InvalidAnnotation, validate_annotation_value
-
+from .ply import lex, yacc
 
 class ParserManager:
     """ This object manages the actual lexer and parser objects providing them
     with state and utility functions.
     """
 
-    def __init__(self, hex_version, encoding, abi_version, tags,
+    def __init__(self, hex_version, encoding, target_abi, tags,
             disabled_features, protected_is_public, include_dirs, sip_module,
             is_strict):
         """ Initialise the manager. """
@@ -77,9 +58,7 @@ class ParserManager:
         # Public state.
         self.tags = tags
 
-        self.spec = Specification(
-                tuple([int(v) for v in abi_version.split('.')]), is_strict,
-                sip_module)
+        self.spec = Specification(target_abi, is_strict, sip_module)
 
         # The module is initially unnamed.
         self.modules = [self.spec.module]
@@ -201,7 +180,7 @@ class ParserManager:
                 else:
                     klass.default_ctor = last_resort
 
-            klass.deprecated = annotations.get('Deprecated', False)
+            klass.deprecated = annotations.get('Deprecated')
 
             if klass.convert_to_type_code is not None and annotations.get('AllowNone', False):
                 klass.handles_none = True
@@ -509,7 +488,7 @@ class ParserManager:
 
         ctor.docstring = docstring
         ctor.gil_action = self._get_gil_action(p, symbol, annotations)
-        ctor.deprecated = annotations.get('Deprecated', False)
+        ctor.deprecated = annotations.get('Deprecated')
 
         if access_specifier is not AccessSpecifier.PRIVATE:
             ctor.kw_args = self._get_kw_args(p, symbol, annotations,
@@ -600,7 +579,7 @@ class ParserManager:
         base_type = EnumBaseType.ENUM
 
         if base_type_s is not None:
-            if self.spec.abi_version < (13, 0):
+            if self.spec.target_abi is not None and self.spec.target_abi < (13, 0):
                 self.parser_error(p, symbol,
                         "/BaseType/ is only supported for ABI v13.0 and later")
 
@@ -611,7 +590,7 @@ class ParserManager:
             elif base_type_s == 'IntEnum':
                 base_type = EnumBaseType.INT_ENUM
             elif base_type_s == 'UIntEnum':
-                base_type = EnumBaseType.UNSIGNED_INT_ENUM
+                base_type = EnumBaseType.UINT_ENUM
             elif base_type_s == 'IntFlag':
                 base_type = EnumBaseType.INT_FLAG
             else:
@@ -748,7 +727,7 @@ class ParserManager:
 
         overload.gil_action = self._get_gil_action(p, symbol, annotations)
         overload.factory = annotations.get('Factory', False)
-        overload.deprecated = annotations.get('Deprecated', False)
+        overload.deprecated = annotations.get('Deprecated')
         overload.new_thread = annotations.get('NewThread', False)
         overload.transfer = self.get_transfer(p, symbol, annotations)
 
@@ -816,6 +795,10 @@ class ParserManager:
                 len_method_code = CodeBlock("Auto-generated",
                         text='            sipRes = (Py_ssize_t)sipCpp->{0}();\n'.format(cpp_name))
 
+                # Note that we can't currently use any %MethodCode because it
+                # does too much (ie. setting sipRes).
+                self.scope.len_cpp_name = cpp_name
+
             len_py_signature = Signature(result=Argument(ArgumentType.SSIZE))
 
             self._add_auto_slot(p, symbol, annotations, '__len__',
@@ -1195,6 +1178,12 @@ class ParserManager:
         # Return any value of the right type.
         return KwArgs.OPTIONAL
 
+    def deprecated(self, p, symbol, instead=None):
+        """ Issue a deprecation message about a symbol. """
+
+        deprecated(f"'{p[symbol]}'", instead=instead, filename=self._sip_file,
+                line_nr=p.lineno(symbol))
+
     def ensure_import(self):
         """ We allow %Modules that are part of a %CompositeModule to be either
         %Imported or %Included.  In the case of the latter we need to adjust
@@ -1298,7 +1287,7 @@ class ParserManager:
 
             return True
 
-        # See if there is a selected qualifier withing range.
+        # See if there is a selected qualifier within range.
         for qual in module.qualifiers:
             if qual.type is QualifierType.TIME and qual.timeline == timeline and qual.name in self.tags:
                 if lower_qual is not None and qual.order < lower_qual.order:
@@ -1534,6 +1523,9 @@ class ParserManager:
         for klass in self._template_arg_classes:
             self.spec.classes.remove(klass)
 
+        # Finalise the target ABI version.
+        self._finalise_target_abi()
+
         return self.spec, self.modules, self._sip_files
 
     def parser_error(self, p, symbol, text):
@@ -1782,7 +1774,7 @@ class ParserManager:
     def validate_mapped_type(self, p, symbol, mapped_type):
         """ Validate a completed mapped type. """
 
-        if self.spec.abi_version >= (13, 0):
+        if self.spec.target_abi is None or self.spec.target_abi >= (13, 0):
             convert_to_us = mapped_type.convert_to_type_code is not None and 'sipUserState' in mapped_type.convert_to_type_code.text
 
             release_us = mapped_type.release_code is not None and 'sipUserState' in mapped_type.release_code.text
@@ -1887,6 +1879,37 @@ class ParserManager:
                 fq_cpp_name, iface_file_type, error_logger, cpp_type=cpp_type,
                 scope=self.scope)
 
+    def _finalise_target_abi(self):
+        """ Finalise the target ABI. """
+
+        target_abi = self.spec.target_abi
+
+        if target_abi is None:
+            major_version = get_latest_version()
+            minor_version = None
+        else:
+            major_version, minor_version = target_abi
+
+        if minor_version is None:
+            minor_version = get_latest_version(major_version)
+
+        # These ABI versions are deprecated because we have deprecated any
+        # arguments to 'throw()' which these versions rely on.
+        if major_version == 12 and minor_version < 9:
+            self._deprecated_target_abi(major_version, minor_version, '12.9')
+
+        if major_version == 13 and minor_version < 1:
+            self._deprecated_target_abi(major_version, minor_version, '13.1')
+
+        self.spec.target_abi = (major_version, minor_version)
+
+    @staticmethod
+    def _deprecated_target_abi(major_version, minor_version, instead):
+        """ Issue a deprecation warning about an old ABI version. """
+
+        deprecated(f"ABI v{major_version}.{minor_version}",
+                instead=f"v{instead} or later")
+
     def _find_member(self, p, symbol, py_name, args, annotations, method_code):
         """ Return (creating if necessary) the member for a Python name. """
 
@@ -2097,8 +2120,8 @@ class ParserManager:
         self.module_state = ModuleState(module, sip_file)
 
         # Get the configuration of the new module.
-        mod_tags, mod_disabled = get_bindings_configuration(
-                self.spec.abi_version[0], sip_file, self._include_dirs)
+        mod_tags, mod_disabled = get_bindings_configuration(self.spec,
+                sip_file, self._include_dirs)
 
         for tag in mod_tags:
             if tag not in self.tags:
