@@ -1,25 +1,10 @@
-# Copyright (c) 2023, Riverbank Computing Limited
-# All rights reserved.
-#
-# This copy of SIP is licensed for use under the terms of the SIP License
-# Agreement.  See the file LICENSE for more details.
-#
-# This copy of SIP may also used under the terms of the GNU General Public
-# License v2 or v3 as published by the Free Software Foundation which can be
-# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-2-Clause
 
+# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
+
+
+from ...exceptions import UserException
+from ...module import parse_abi_version
 
 from ..scoped_name import ScopedName
 from ..specification import (AccessSpecifier, Argument, ArgumentType,
@@ -90,6 +75,7 @@ def p_statement(p):
         | license
         | mapped_type
         | mapped_type_template
+        | minimum_abi_version
         | module
         | module_code
         | module_header_code
@@ -1009,6 +995,41 @@ def p_mapped_type_function(p):
     pm.validate_function(p, 1, overload)
 
 
+# %MinimumABIVersion ##########################################################
+
+def p_minimum_abi_version(p):
+    """minimum_abi_version : MinimumABIVersion STRING
+        | MinimumABIVersion begin_args '(' version '=' STRING end_args ')'"""
+
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    symbol = 2 if len(p) == 3 else 6
+
+    try:
+        min_major_version, min_minor_version = parse_abi_version(p[symbol])
+    except UserException as e:
+        pm.parser_error(p, symbol, e.text)
+        return
+
+    if pm.spec.target_abi is None:
+        pm.spec.target_abi = (min_major_version, min_minor_version)
+    else:
+        major_version, minor_version = pm.spec.target_abi
+
+        if major_version != min_major_version:
+            pm.parser_error(p, symbol,
+                    f"v{min_major_version} is specified but v{major_version} is being targeted")
+        elif min_minor_version is not None:
+            if minor_version is None:
+                pm.spec.target_abi = (min_major_version, min_minor_version)
+            elif min_minor_version is not None and minor_version < min_minor_version:
+                pm.parser_error(p, symbol,
+                        f"v{min_major_version}.{min_minor_version} is the minimum ABI version but v{major_version}.{minor_version} is being targeted")
+
+
 # %ModuleHeaderCode ###########################################################
 
 def p_module_header_code(p):
@@ -1077,6 +1098,7 @@ def p_module(p):
         elif pm.c_bindings != c_bindings:
             pm.parser_error(p, 1, "cannot mix 'C' and 'C++' modules")
 
+    # Deprecate and remove when Python v3.12 is no longer supported.
     if 'py_ssize_t_clean' in args:
         module.py_ssize_t_clean = args['py_ssize_t_clean']
 
@@ -1663,6 +1685,9 @@ def p_pod_type(p):
         else:
             p[0] = _UNSIGNED_MAP[p[2]]
     else:
+        if p[1] == 'SIP_SSIZE_T':
+            p.parser.pm.deprecated(p, 1, instead="'Py_ssize_t'")
+
         p[0] = _ONE_WORD_MAP[p[1]]
 
 
@@ -1942,10 +1967,21 @@ def p_class_line(p):
         | type_code
         | type_header_code
         | type_hint_code
-        | BIGetReadBufferCode CODE_BLOCK
-        | BIGetWriteBufferCode CODE_BLOCK
-        | BIGetSegCountCode CODE_BLOCK
-        | BIGetCharBufferCode CODE_BLOCK"""
+        | deprecated_code_directives CODE_BLOCK"""
+
+
+def p_deprecated_code_directives(p):
+    """deprecated_code_directives : BIGetReadBufferCode
+        | BIGetWriteBufferCode
+        | BIGetSegCountCode
+        | BIGetCharBufferCode"""
+
+    pm = p.parser.pm
+
+    if pm.skipping:
+        return
+
+    pm.deprecated(p, 1)
 
 
 # The ctor annotations.
@@ -2781,6 +2817,7 @@ def p_opt_exceptions(p):
 
     if p[1] == 'throw':
         p[0] = p[3]
+        p.parser.pm.deprecated(p, 1)
     elif p[1] == 'noexcept':
         p[0] = ThrowArguments()
     else:
@@ -2791,7 +2828,7 @@ def p_opt_exception_list(p):
     """opt_exception_list : exception_list
         | empty"""
 
-    p[0] = ThrowArguments(arguments=[]) if p[1] is None else p[1]
+    p[0] = ThrowArguments() if p[1] is None else p[1]
 
 
 def p_exception_list(p):
@@ -3127,6 +3164,15 @@ def p_variable(p):
     annos_symbol = 3
     body = p[4]
 
+    # We don't currently support /AllowNone/ and /DisallowNone/ for variables.
+    # Generated variable setters support allow None for some (pointer) types
+    # but not others.  For pointer types we explicity disallow None.  This
+    # doesn't affect the generated code but does mean that type hints match
+    # those from previous versions.  This should be changed to properly support
+    # the annotations.
+    if len(type.derefs) != 0:
+        type.disallow_none = True
+
     annotations = p[annos_symbol]
 
     pm.check_annotations(p, annos_symbol, "variable", _VARIABLE_ANNOTATIONS)
@@ -3206,10 +3252,15 @@ def p_annotation(p):
     """annotation : NAME
         | NAME '=' annotation_value"""
 
+    p[0] = {}
+
+    if p.parser.pm.skipping:
+        return
+
     value = None if len(p) == 2 else p[3]
     value = p.parser.pm.validate_annotation(p, 1, value)
 
-    p[0] = {p[1]: value}
+    p[0][p[1]] = value
 
 
 def p_annotation_value(p):
