@@ -1,24 +1,6 @@
-# Copyright (c) 2023, Riverbank Computing Limited
-# All rights reserved.
-#
-# This copy of SIP is licensed for use under the terms of the SIP License
-# Agreement.  See the file LICENSE for more details.
-#
-# This copy of SIP may also used under the terms of the GNU General Public
-# License v2 or v3 as published by the Free Software Foundation which can be
-# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-2-Clause
+
+# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
 
 
 
@@ -169,7 +151,7 @@ def resolve(spec, modules):
 
         # Include the %TypeHeaderCode for exceptions defined in the main
         # module.
-        if spec.abi_version >= (13, 1) or (spec.abi_version >= (12, 9) and spec.abi_version < (13, 0)):
+        if spec.target_abi >= (13, 1) or (spec.target_abi >= (12, 9) and spec.target_abi < (13, 0)):
             if exception_mod is spec.module:
                 append_iface_file(spec.module.used, exception.iface_file)
 
@@ -564,7 +546,7 @@ def _move_global_slot(spec, global_slot, error_log):
             # correctly (ie. 'E.M == E.M' works as expected).  However if there
             # is another equality operator defined then it will fail so we have
             # to explicitly inject the comparison.
-            if spec.abi_version < (13, 0) and arg0.type is ArgumentType.ENUM and arg_member.py_slot is PySlot.EQ and not is_second:
+            if spec.target_abi < (13, 0) and arg0.type is ArgumentType.ENUM and arg_member.py_slot is PySlot.EQ and not is_second:
                 inject_equality_slot = True
 
         # Move the overload to the end of the destination list.
@@ -667,8 +649,8 @@ def _set_mro(spec, klass, error_log, seen=None):
     if klass.scope is not None:
         _set_mro(spec, klass.scope, error_log, seen=seen)
 
-        if klass.scope.deprecated:
-            klass.deprecated = True
+        if klass.scope.deprecated and not klass.deprecated:
+            klass.deprecated = klass.scope.deprecated
 
     if klass.iface_file.type is IfaceFileType.CLASS:
         # The first thing is itself.
@@ -705,8 +687,8 @@ def _set_mro(spec, klass, error_log, seen=None):
                 if klass.iface_file.module is spec.module:
                     superklass_mro.iface_file.needed = True
 
-                if superklass_mro.deprecated:
-                    klass.deprecated = True
+                if superklass_mro.deprecated and not klass.deprecated:
+                    klass.deprecated = superklass_mro.deprecated
 
                 # If the super-class is a QObject sub-class then this one is as
                 # well.
@@ -834,8 +816,9 @@ def _resolve_ctors(spec, klass, error_log):
                                     klass.iface_file.fq_cpp_name))
                     break
 
-        if klass.deprecated:
-            ctor.deprecated = True
+
+        if klass.deprecated and not ctor.deprecated :
+            ctor.deprecated = klass.deprecated   
 
 
 def _transform_casts(spec, klass, error_log):
@@ -890,9 +873,9 @@ def _add_default_copy_ctor(klass):
     signature = Signature(args=[arg], result=result)
     ctor = Constructor(AccessSpecifier.PUBLIC, py_signature=signature,
             cpp_signature=signature)
- 
-    if klass.deprecated:
-        ctor.deprecated = True
+
+    if klass.deprecated and not ctor.deprecated :
+        ctor.deprecated = klass.deprecated
 
     if not klass.is_abstract:
         klass.can_create = True
@@ -937,8 +920,9 @@ def _resolve_scope_overloads(spec, overloads, error_log, final_checks,
                     break
 
         if isinstance(scope, WrappedClass):
-            if scope.deprecated:
-                overload.deprecated = True
+
+            if scope.deprecated and not overload.deprecated :
+                overload.deprecated = scope.deprecated
 
             if overload.is_abstract:
                 scope.is_abstract = True
@@ -1265,7 +1249,7 @@ def _resolve_func_types(spec, mod, scope, overload, error_log, final_checks):
 
     # These slots must return Py_ssize_t.
     if is_ssize_return_slot(overload.common.py_slot):
-        if spec.abi_version >= (13, 0):
+        if spec.target_abi >= (13, 0):
             required_types = (ArgumentType.SSIZE, )
         else:
             required_types = (ArgumentType.SSIZE, ArgumentType.INT)
@@ -1288,7 +1272,7 @@ def _resolve_func_types(spec, mod, scope, overload, error_log, final_checks):
 
     # These slots must return Py_hash_t.
     if is_hash_return_slot(overload.common.py_slot):
-        if spec.abi_version >= (13, 0):
+        if spec.target_abi >= (13, 0):
             required_type = ArgumentType.HASH
             required_type_name = 'Py_hash_t'
         else:
@@ -2052,7 +2036,7 @@ def _iface_files_are_used_by_overload(spec, used, overload, need_types=False):
                 need_types=need_types)
 
     # Don't bother with %TypeHeaderCode from %Exception for later ABI versions.
-    if spec.abi_version >= (13, 1) or (spec.abi_version >= (12, 9) and spec.abi_version < (13, 0)):
+    if spec.target_abi >= (13, 1) or (spec.target_abi >= (12, 9) and spec.target_abi < (13, 0)):
         return
 
     throw_args = overload.throw_args
