@@ -1,24 +1,6 @@
-# Copyright (c) 2023, Riverbank Computing Limited
-# All rights reserved.
-#
-# This copy of SIP is licensed for use under the terms of the SIP License
-# Agreement.  See the file LICENSE for more details.
-#
-# This copy of SIP may also used under the terms of the GNU General Public
-# License v2 or v3 as published by the Free Software Foundation which can be
-# found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
-#
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
-# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-# POSSIBILITY OF SUCH DAMAGE.
+# SPDX-License-Identifier: BSD-2-Clause
+
+# Copyright (c) 2025 Phil Thompson <phil@riverbankcomputing.com>
 
 
 import collections
@@ -34,21 +16,25 @@ import warnings
 from .abstract_builder import AbstractBuilder
 from .abstract_project import AbstractProject
 from .bindings import Bindings
+from .buildable import BuildableFromSources
 from .configurable import Configurable, Option
-from .exceptions import deprecated, UserException
-from .module import resolve_abi_version
+from .exceptions import UserException
+from .module import get_source_version_range, parse_abi_version
 from .py_versions import OLDEST_SUPPORTED_MINOR
 from .pyproject import PyProjectException, PyProjectOptionException
 
 
+# The macOS minimum version of the latest supported version of Python.  v10.13
+# is the minimum version required by Python v3.13.
+MACOS_MIN_OS_MAJOR = 10
+MACOS_MIN_OS_MINOR = 13
+
+
 class Project(AbstractProject, Configurable):
     """ Encapsulate a project containing one or more sets of bindings. """
 
     # The configurable options.
     _options = (
-        # The minimum required ABI version of the sip module.
-        Option('abi_version'),
-
         # The callable that will return an Bindings instance.  This is used for
         # bindings implicitly defined in the .toml file.
         Option('bindings_factory'),
@@ -117,6 +103,8 @@ class Project(AbstractProject, Configurable):
                 help="enable verbose progress messages"),
         Option('name', help="the name used in sdist and wheel file names",
                 metavar="NAME", tools=['sdist', 'wheel']),
+        Option('abi_version', help="The ABI to generate code for",
+                metavar="M[.N]"),
         Option('build_dir', help="the build directory", metavar="DIR"),
         Option('build_tag', help="the build tag to be used in the wheel name",
                 metavar="TAG", tools=['wheel']),
@@ -169,11 +157,29 @@ class Project(AbstractProject, Configurable):
         self.buildables = []
         self.installables = []
 
+        # The provisional target ABI is that specified by abi-version in
+        # pyproject.toml (or on the command line).  Note that, for historical
+        # reasons, we may not have the source of the older minor versions in
+        # which case we will build against the oldest we do have.
+        self.target_abi = None
+        self._build_abi = None
+
         self._metadata_overrides = None
         self._temp_build_dir = None
 
         self.initialise_options(kwargs)
 
+    @property
+    def all_modules_use_limited_abi(self):
+        """ True if all modules in the project use the limited ABI. """
+
+        for buildable in self.buildables:
+            if isinstance(buildable, BuildableFromSources):
+                if not buildable.uses_limited_api:
+                    return False
+
+        return True
+
     def apply_nonuser_defaults(self, tool):
         """ Set default values for non-user options that haven't been set yet.
         """
@@ -264,6 +270,20 @@ class Project(AbstractProject, Configurable):
 
         return wheel_file
 
+    @property
+    def build_abi(self):
+        """ The ABI version to use to build against.  This may be later than
+        the target version if we don't have the source of the target version.
+        It must not be called before the target ABI has been finalised.
+        """
+
+        if self._build_abi is None:
+            major_version, minor_version = self.target_abi
+            oldest_source, _ = get_source_version_range(major_version)
+            self._build_abi = (major_version, max(minor_version, oldest_source))
+
+        return self._build_abi
+
     def get_bindings_dir(self):
         """ Return the name of the 'bindings' directory relative to the
         eventual target directory.
@@ -326,20 +346,42 @@ class Project(AbstractProject, Configurable):
 
         platform_tag = sysconfig.get_platform()
 
-        if self.py_platform == 'darwin' and self.minimum_macos_version:
+        if self.py_platform == 'darwin':
             # We expect a three part tag so leave anything else unchanged.
             parts = platform_tag.split('-')
             if len(parts) == 3:
-                min_major = int(self.minimum_macos_version[0])
-                min_minor = int(self.minimum_macos_version[1])
+                version = parts[1]
 
-                # For arm64 binaries enforce a valid minimum macOS version.
+                if '.' in version:
+                    min_major, min_minor = version.split('.')
+                    min_major = int(min_major)
+                    min_minor = int(min_minor)
+                else:
+                    min_major = int(version)
+                    min_minor = 0
+
+                # Use the user supplied value if it is later than the platform
+                # value.
+                if self.minimum_macos_version:
+                    user_min_major = int(self.minimum_macos_version[0])
+                    user_min_minor = int(self.minimum_macos_version[1])
+
+                    if (min_major, min_minor) < (user_min_major, user_min_minor):
+                        min_major = user_min_major
+                        min_minor = user_min_minor
+
+                # Enforce a minimum macOS version for limited ABI projects.
+                if self.all_modules_use_limited_abi and (min_major, min_minor) < (MACOS_MIN_OS_MAJOR, MACOS_MIN_OS_MINOR):
+                    min_major = MACOS_MIN_OS_MAJOR
+                    min_minor = MACOS_MIN_OS_MINOR
+
+                # Enforce a minimum macOS version for arm64 binaries.
                 if parts[2] == 'arm64' and min_major < 11:
                     min_major = 11
                     min_minor = 0
 
-                parts[1] = '{}.{}'.format(min_major, min_minor)
-
+                # Reassemble the tag.
+                parts[1] = f'{min_major}.{min_minor}'
                 platform_tag = '-'.join(parts)
 
         elif self.py_platform == 'linux' and self.manylinux:
@@ -352,7 +394,7 @@ class Project(AbstractProject, Configurable):
                     major, minor = 2, 5
 
                 parts[0] = 'manylinux'
-                parts.insert(1, '{}.{}'.format(major, minor))
+                parts.insert(1, f'{major}.{minor}')
 
                 platform_tag = '-'.join(parts)
 
@@ -378,10 +420,10 @@ class Project(AbstractProject, Configurable):
             if rd.split()[0] == sip_project_name:
                 return []
 
-        next_abi_major = int(self.abi_version.split('.')[0]) + 1
+        abi_major, abi_minor = self.target_abi
+        next_abi_major = abi_major + 1
 
-        return ['{} (>={}, <{})'.format(sip_project_name, self.abi_version,
-                next_abi_major)]
+        return [f'{sip_project_name} (>={abi_major}.{abi_minor}, <{next_abi_major})']
 
     def get_sip_distinfo_command_line(self, sip_distinfo, inventory,
             generator=None, wheel_tag=None, generator_version=None):
@@ -664,19 +706,9 @@ class Project(AbstractProject, Configurable):
                     "Python v{}.{} is not supported".format(
                             self.py_major_version, self.py_minor_version))
 
-        # Get the supported ABI version.  (The actual version may have a later
-        # minor version.)
-        self.abi_version = resolve_abi_version(self.abi_version, module=False)
-
-        # These ABI versions are deprecated because we have deprecated any
-        # arguments to 'throw()' which these versions rely on.
-        abi_version = tuple([int(v) for v in self.abi_version.split('.')])
-
-        if abi_version < (12, 9):
-            self._deprecated_abi_version('12.9')
-
-        if abi_version == (13, 0):
-            self._deprecated_abi_version('13.1')
+        # Get the provisional ABI version to target.
+        if self.abi_version:
+            self.target_abi = parse_abi_version(self.abi_version)
 
         # Checks for standalone projects.
         if tool in Option.BUILD_TOOLS and not self.sip_module:
@@ -770,11 +802,6 @@ class Project(AbstractProject, Configurable):
 
         return int(parts[0]), int(parts[1])
 
-    def _deprecated_abi_version(self, instead):
-        """ Issue a deprecation warning about an old ABI version. """
-
-        deprecated(f"ABI v{self.abi_version}", instead=f"v{instead} or later")
-
     def _enable_disable_bindings(self):
         """ Check the enabled bindings are valid and remove any disabled ones.
         """
